/*
 * Copyright 2013 Mark Slater
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.
 */

package net.sourceforge.xazzle.xhtml;

import net.sourceforge.writexml.Attribute;
import net.sourceforge.writexml.Attributes;
import net.sourceforge.writexml.TagName;

import static java.util.Arrays.asList;
import static net.sourceforge.writexml.Attribute.attribute;
import static net.sourceforge.writexml.AttributeName.attributeName;
import static net.sourceforge.writexml.AttributeValue.attributeValue;
import static net.sourceforge.writexml.Attributes.attributes;
import static net.sourceforge.writexml.TagName.tagName;

public final class AnchorTag extends TabbableStandardXhtmlTag<AnchorTag, XhtmlTag> implements BodyElement<DoesNotContainFormTag, TagScope.Inline> {
    private AnchorTag(final TagName tagName, final Attributes attributes, final Iterable<? extends XhtmlTag> children) {
        super(tagName, attributes, children);
    }

    static AnchorTag anchorTag(final BodyElement<?, TagScope.InlineExceptAnchor>... children) {
        return anchorTag(asList(children));
    }

    static AnchorTag anchorTag(final Iterable<? extends BodyElement<?, TagScope.InlineExceptAnchor>> children) {
        return new AnchorTag(tagName("a"), attributes(), children);
    }

    @Override
    AnchorTag makeANewOne(final Attribute attribute) {
        return new AnchorTag(tagName, attributes.append(attribute), children);
    }

    public AnchorTag withCharacterSet(final CharacterSet characterSet) {
        return makeANewOne(attribute(attributeName("charset"), attributeValue(characterSet.asString()), NAMESPACE));
    }

    public AnchorTag withHref(final Href href) {
        return makeANewOne(attribute(attributeName("href"), attributeValue(href.asString()), NAMESPACE));
    }

    public AnchorTag withHrefLanguage(final LanguageCode languageCode) {
        return makeANewOne(attribute(attributeName("hreflang"), attributeValue(languageCode.asString()), NAMESPACE));
    }

    public AnchorTag withRelationships(final Relationships relationships) {
        return makeANewOne(attribute(attributeName("rel"), attributeValue(relationships.asString()), NAMESPACE));
    }

    public AnchorTag withRelationships(final Relationship relationship, final Relationship... relationships) {
        return withRelationships(Relationships.relationships(relationship, relationships));
    }

    public AnchorTag withReverseLinks(final Relationships relationships) {
        return makeANewOne(attribute(attributeName("rev"), attributeValue(relationships.asString()), NAMESPACE));
    }

    public AnchorTag withOnFocus(final EventHandler eventHandler) {
        return makeANewOne(attribute(attributeName("onfocus"), attributeValue(eventHandler.asString()), NAMESPACE));
    }

    public AnchorTag withOnBlur(final EventHandler eventHandler) {
        return makeANewOne(attribute(attributeName("onblur"), attributeValue(eventHandler.asString()), NAMESPACE));
    }

    public AnchorTag withShape(final Shape shape) {
        return makeANewOne(attribute(attributeName("shape"), attributeValue(shape.asString()), NAMESPACE));
    }

    public AnchorTag withCoordinates(final Coordinates coordinates) {
        return makeANewOne(attribute(attributeName("coords"), attributeValue(coordinates.asString()), NAMESPACE));
    }

    public AnchorTag withName(final Name name) {
        return makeANewOne(attribute(attributeName("name"), attributeValue(name.asString()), NAMESPACE));
    }
}
