/*
 * Copyright 2013 Mark Slater
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.
 */

package net.sourceforge.xazzle.xhtml;

import net.sourceforge.writexml.Attribute;
import net.sourceforge.writexml.Attributes;

import java.util.Collections;

import static net.sourceforge.writexml.Attribute.attribute;
import static net.sourceforge.writexml.AttributeName.attributeName;
import static net.sourceforge.writexml.AttributeValue.attributeValue;
import static net.sourceforge.writexml.Attributes.attributes;
import static net.sourceforge.writexml.TagName.tagName;

public final class AreaTag extends TabbableStandardXhtmlTag<AreaTag, XhtmlTag> implements MapElement<DoesNotContainFormTag> {

    private AreaTag(final Attributes attributes) {
        super(tagName("area"), attributes, Collections.<XhtmlTag>emptyList());
    }

    static AreaTag areaTag(final AlternateText alternateText) {
        return new AreaTag(attributes(attribute(attributeName("alt"), attributeValue(alternateText.asString()), NAMESPACE)));
    }

    @Override
    AreaTag makeANewOne(Attribute attribute) {
        return new AreaTag(attributes.append(attribute));
    }

    public AreaTag withHref(final Href href) {
        return makeANewOne(attribute(attributeName("href"), attributeValue(href.asString()), NAMESPACE));
    }

    public AreaTag withOnFocus(final EventHandler eventHandler) {
        return makeANewOne(attribute(attributeName("onfocus"), attributeValue(eventHandler.asString()), NAMESPACE));
    }

    public AreaTag withOnBlur(final EventHandler eventHandler) {
        return makeANewOne(attribute(attributeName("onblur"), attributeValue(eventHandler.asString()), NAMESPACE));
    }

    public AreaTag withoutHref() {
        return makeANewOne(attribute(attributeName("nohref"), attributeValue("nohref"), NAMESPACE));
    }

    public AreaTag withShape(final Shape shape) {
        return makeANewOne(attribute(attributeName("shape"), attributeValue(shape.asString()), NAMESPACE));
    }

    public AreaTag withCoordinates(final Coordinates coordinates) {
        return makeANewOne(attribute(attributeName("coords"), attributeValue(coordinates.asString()), NAMESPACE));
    }
}
