/*
 * Copyright 2013 Mark Slater
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.
 */

package net.sourceforge.xazzle.xhtml;

import net.sourceforge.writexml.Attribute;
import net.sourceforge.writexml.AttributeName;
import net.sourceforge.writexml.Attributes;

import static java.util.Arrays.asList;
import static net.sourceforge.writexml.Attributes.attributes;
import static net.sourceforge.writexml.TagName.tagName;
import static net.sourceforge.xazzle.xhtml.IterableUtils.cons;

public final class HeadTag extends BasicXhtmlTag<HeadTag, XhtmlTag> {

    private HeadTag(final Attributes attributes, final Iterable<? extends XhtmlTag> children) {
        super(tagName("head"), attributes, children);
    }

    static HeadTag headTag(final TitleTag titleTag, final HeadChild... children) {
        return headTag(titleTag, asList(children));
    }

    static HeadTag headTag(final TitleTag titleTag, final Iterable<HeadChild> children) {
        return new HeadTag(attributes(), cons(titleTag, CollectionUtils.<XhtmlTag>newList(children)));
    }

    static HeadTag headTag(final BaseTag baseTag, final TitleTag titleTag, final HeadChild... children) {
        return headTag(baseTag, titleTag, asList(children));
    }

    static HeadTag headTag(final BaseTag baseTag, final TitleTag titleTag, final Iterable<HeadChild> children) {
        return new HeadTag(attributes(), cons(baseTag, cons(titleTag, CollectionUtils.<XhtmlTag>newList(children))));
    }

    @Override
    HeadTag makeANewOne(final Attribute attribute) {
        return new HeadTag(attributes.append(attribute), children);
    }

    public HeadTag withProfile(final Profile profile) {
        return new HeadTag(attributes.append(Attribute.attribute(AttributeName.attributeName("profile"), net.sourceforge.writexml.AttributeValue.attributeValue(profile.asString()), NAMESPACE)), children);
    }
}
