/*
 * Copyright 2013 Mark Slater
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.
 */

package net.sourceforge.xazzle.xhtml;

import net.sourceforge.writexml.Attribute;
import net.sourceforge.writexml.Attributes;
import net.sourceforge.writexml.TagName;

import java.util.Collections;

import static java.util.Arrays.asList;
import static net.sourceforge.writexml.Attributes.attributes;
import static net.sourceforge.writexml.TagName.tagName;

public final class InlineTag extends StandardXhtmlTag<InlineTag, XhtmlTag> implements BodyElement<DoesNotContainFormTag, TagScope.InlineExceptAnchor> {

    private InlineTag(final TagName tagName, final Attributes attributes, final Iterable<? extends XhtmlTag> children) {
        super(tagName, attributes, children);
    }

    static InlineTag codeTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return codeTag(asList(children));
    }

    static InlineTag codeTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("code"), children);
    }

    static InlineTag sampleCodeTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return sampleCodeTag(asList(children));
    }

    static InlineTag sampleCodeTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("samp"), children);
    }

    static InlineTag abbreviationTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return abbreviationTag(asList(children));
    }

    static InlineTag abbreviationTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("abbr"), children);
    }

    static InlineTag superscriptTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return superscriptTag(asList(children));
    }

    static InlineTag superscriptTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("sup"), children);
    }

    static InlineTag keyboardTextTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return keyboardTextTag(asList(children));
    }

    static InlineTag keyboardTextTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("kbd"), children);
    }

    static InlineTag strongTextTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return strongTextTag(asList(children));
    }

    static InlineTag strongTextTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("strong"), children);
    }

    static InlineTag citationTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return citationTag(asList(children));
    }

    static InlineTag citationTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("cite"), children);
    }

    static InlineTag definitionTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return definitionTag(asList(children));
    }

    static InlineTag definitionTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("dfn"), children);
    }

    static InlineTag variableTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return variableTag(asList(children));
    }

    static InlineTag variableTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("var"), children);
    }

    static InlineTag spanTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return spanTag(asList(children));
    }

    static InlineTag spanTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("span"), children);
    }

    static InlineTag emphasisedTextTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return emphasisedTextTag(asList(children));
    }

    static InlineTag emphasisedTextTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("em"), children);
    }

    static InlineTag acronymTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return acronymTag(asList(children));
    }

    static InlineTag acronymTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("acronym"), children);
    }

    static InlineTag subscriptTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return subscriptTag(asList(children));
    }

    static InlineTag subscriptTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("sub"), children);
    }

    static InlineTag teletypeTextTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return teletypeTextTag(asList(children));
    }

    static InlineTag teletypeTextTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("tt"), children);
    }

    static InlineTag bigTextTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return bigTextTag(asList(children));
    }

    static InlineTag bigTextTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("big"), children);
    }

    static InlineTag boldTextTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return boldTextTag(asList(children));
    }

    static InlineTag boldTextTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("b"), children);
    }

    static InlineTag italicTextTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return italicTextTag(asList(children));
    }

    static InlineTag italicTextTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("i"), children);
    }

    static InlineTag smallTextTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return smallTextTag(asList(children));
    }

    static InlineTag smallTextTag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("small"), children);
    }

    static InlineTag bidirectionalOverrideTag(final TextDirection textDirection, final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return bidirectionalOverrideTag(textDirection, asList(children));
    }

    static InlineTag bidirectionalOverrideTag(final TextDirection textDirection, final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return inlineTag(tagName("bdo"), children).withDirection(textDirection);
    }

    static InlineTag lineBreakTag() {
        return inlineTag(tagName("br"), Collections.<BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>>emptyList());
    }

    private static InlineTag inlineTag(final TagName name, final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return new InlineTag(name, attributes(), children);
    }

    @Override
    InlineTag makeANewOne(final Attribute attribute) {
        return new InlineTag(tagName, attributes.append(attribute), children);
    }
}
