/*
 * Copyright 2013 Mark Slater
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.
 */

package net.sourceforge.xazzle.xhtml;

import net.sourceforge.writexml.Attribute;
import net.sourceforge.writexml.Attributes;
import net.sourceforge.writexml.Namespace;

import static java.util.Arrays.asList;
import static net.sourceforge.writexml.Attribute.attribute;
import static net.sourceforge.writexml.AttributeName.attributeName;
import static net.sourceforge.writexml.AttributeValue.attributeValue;
import static net.sourceforge.writexml.Attributes.attributes;
import static net.sourceforge.writexml.TagName.tagName;

public final class MapTag<T extends MayContainFormTag> extends FixedIdBaseXhtmlTag<MapTag<T>, XhtmlTag> implements BodyElement<T, TagScope.Block> {

    private MapTag(final Attributes attributes, final Iterable<? extends XhtmlTag> children) {
        super(tagName("map"), attributes, children);
    }

    static <T extends MayContainFormTag> MapTag<T> mapTag(final Id id, final MapElement<? extends T> mapElement, final MapElement<? extends T>... mapElements) {
        return mapTag(id, mapElement, asList(mapElements));
    }

    static <T extends MayContainFormTag> MapTag<T> mapTag(final Id id, final MapElement<? extends T> mapElement, final Iterable<? extends MapElement<? extends T>> mapElements) {
        return new MapTag<>(attributes(attribute(attributeName("id"), attributeValue(id.asString()), NAMESPACE)), IterableUtils.cons(mapElement, mapElements));
    }

    @Override
    MapTag<T> makeANewOne(final Attribute attribute) {
        return new MapTag<>(attributes.append(attribute), children);
    }

    public MapTag<T> withName(final Name name) {
        return makeANewOne(attribute(attributeName("name"), attributeValue(name.asString()), NAMESPACE));
    }

    public MapTag<T> withDirection(final TextDirection textDirection) {
        return makeANewOne(attribute(attributeName("dir"), attributeValue(textDirection.asString()), NAMESPACE));
    }

    public MapTag<T> withLanguage(final LanguageCode languageCode) {
        return makeANewOne(attribute(attributeName("lang"), attributeValue(languageCode.asString()), Namespace.XML));
    }

    public final MapTag<T> withClass(final ClassName className) {
        return makeANewOne(attribute(attributeName("class"), attributeValue(className.asString()), NAMESPACE));
    }

    public final MapTag<T> withStyle(final Style style) {
        return makeANewOne(attribute(attributeName("style"), attributeValue(style.asString()), NAMESPACE));
    }

    public final MapTag<T> withTitle(final Title title) {
        return makeANewOne(attribute(attributeName("title"), attributeValue(title.asString()), NAMESPACE));
    }

}
